import pygame
from pygame.constants import *

import data
import engine
import items
import rpg
import playing

class CharacterState(engine.State):
    def __init__(self, parent, creation=True, page=0):
        engine.State.__init__(self, parent.driver)
        self.page = page
        self.parent = parent
        self._active = None
        self.creation=creation
        self.aboutToAssign = None
        
        prefix = ""
        suffix = ''
        if creation: 
            prefix="Enter your "
        else:
            suffix = parent.pc.name
        namePrompt = engine.Label("%sName: %s"% (prefix,suffix))
        namePrompt.rect.topleft = [ 10, 10 ]
        self.add(namePrompt)
        
        nameLabel = engine.Label("")
        nameLabel.rect.topleft = [namePrompt.rect.right, 10]
        self.add(nameLabel)
        self.nameLabel = nameLabel
        self.name = ''
                
        if creation:
            intro =("\"And thus did the warrior enter the Tower To Heaven, to",
                "do battle with the spirits of those who had failed before,",
                "and ascend into godhood...\"")
            y = (self.driver.screen.h / 4) * 3
            for line in intro:
                introLabel = engine.Label(line)
                introLabel.rect.center = [ self.driver.screen.w / 2, y]
                self.add(introLabel)
                y += introLabel.rect.h + 3
                
            controls = ( "Arrow keys move your character.  'g' grabs loot,",
                         "'i' accesses your inventory, and 's' climbs stairs.",
                         "'q' will save and quit, while 'escape' simply quits.",
                         "'c' shows you your character sheet.",
                         "Numbers 1,2,3,4,5,6,7,8,9,0 activate your hotkeys.",
                         "You'll automatically fight anything that attacks you",
                         " ",
                         "(HINT: One of the first things you'll want to do is",
                         " assign your healing potions to hotkeys. To do this,",
                         " hit 'i' then 'a' to assign, then the number of the ",
                         " potion.  This is also how you use the skills on",
                         " your character sheet)")
            
            y = self.nameLabel.rect.bottom + 100
            for line in controls:
                introLabel = engine.Label(line)
                introLabel.rect.topleft = [ 20 , y]
                self.add(introLabel)
                y += introLabel.rect.h + 3

            beginButton = engine.Button("Press enter or click here to begin",
                                         self.begin)
            beginButton.rect.bottomleft = [ 3, self.driver.screen.h - 3 ]
        else:
            
            self.reportLabel = engine.Label(" ")
            self.reportLabel.color = (255,255,255)
            self.reportLabel.rect.center = [50, 50]
            self.add(self.reportLabel)
            
            y = self.nameLabel.rect.bottom + 100
            
            skillLabel = engine.Label("Skills:")
            skillLabel.rect.topleft = [ 20, y ]
            y += skillLabel.rect.height + 3
            
            self.skillLabels = []
            for index, x in enumerate(data.hotkeys):
                name = ""
                index += page * 10
                if index < len(self.parent.pc.skills):
                    name = self.parent.pc.skills[index]
                label = engine.Button("%d: %s" % (x, name), self.skill, str(x))
                label.rect.topleft = [ 20, y ]
                self.skillLabels.append(label)
                self.add(label)
                y = label.rect.bottom + 3
            
            prev = engine.Button("(p)rev", self.prev,'p')
            prev.rect.topleft = [20, y]
            self.add(prev)
            
            next = engine.Button("(n)ext", self.next, 'n')
            next.rect.topleft = [prev.rect.right + 3, y ]
            self.add(next)
            y += next.rect.height + 3
            
            beginButton = engine.Button("(c)lose", self.close,'c')
            beginButton.rect.bottomleft = [ 3, self.driver.screen.h - 3 ]
            
            x = beginButton.rect.right + 3
            self.assignButton = engine.Button("(a)ssign skill to hotkey", 
                                              self.assign, 'a')
            self.assignButton.rect.topleft = [x, y ]
            self.add(self.assignButton)
                     
        self.add(beginButton)
        
    def activate(self):
        if self.aboutToAssign != None:
            assigned = self.which.chosen
            if assigned != None:
                skill = self.parent.pc.skills[self.aboutToAssign]
                self.parent.hotkeys[assigned] = skill
                data = (skill, (assigned + 1) % 10 )
                self.reportLabel.text = "Assigned %s to hotkey %d" % data
        
    def assign(self, button):
        active = self.active
        if active and active in self.skillLabels:
            index = self.getPickedSkill(active)
            if index != None:
                self.aboutToAssign = index
                self.which = ChooseFromTenState(self, self.parent.hotkeys,
                                                "Choose which hotkey to assign")
                self.driver.push(self.which)
            else:
                self.active = None
        else:
            self.active = button
        
    def begin(self,button=None):
        pc = rpg.PC("stan.png")
        if not self.name: self.name="Stan"

        pc.name = self.name
        startWeapon = rpg.Weapon("Pointy Stick",5,0)
        startArmor = rpg.Armor("Peasant's Outfit", rpg.Armor.TORSO, 1)
        pc.equip(startWeapon)
        pc.equip(startArmor)
        
        health = items.HEALTH_10
        health.amount = 10
        pc.inventory.append(health)
        
        pc.levelup()
        
        pgs = playing.PlayingGameState(self.driver,pc)
        self.close()
        self.driver.push(pgs)
        
    def close(self,button=None):
        self.driver.pop()
        
    def getActive(self):
        return self._active
        
    def getPickedSkill(self, active):
        index = int(active.text[0])
        if index == 0: index = 10
        index -= 1
        index += self.page * 10
        if index < len(self.parent.pc.skills):
            return index
        else:
            return None
        
    def keyEvent(self, event):
        if(self.creation):
            key = event.key
            if event.type == KEYDOWN:
                if key == K_BACKSPACE:
                    if len(self.name) > 0:
                        self.name = self.name[:-1]
                elif key == K_RETURN:
                    self.begin()
                else:
                    self.name = self.name + event.unicode
                self.nameLabel.text = self.name
        else:
            engine.State.keyEvent(self, event)
                
    def next(self, button):
        self.page += 1
        self.refresh()
        
    def prev(self, button):
        self.page = max(0, self.page-1)
        self.refresh()
        
    def refresh(self):
        self.driver.pop()
        self.driver.push(CharacterState(self.parent,False,self.page))     
        
    def skill(self, button):
        oldActive = self._active
        self.active = button
        if oldActive == self.assignButton:
            self.assign(button)
        
    def setActive(self, button):
        if self._active:
            self._active.background = None
        self._active = button
        if button: 
            button.background = ( 64, 64, 64 )
            
    active = property(getActive, setActive)

class ChooseFromTenState(engine.State):
    def __init__(self, pgs, ten,title):
        engine.State.__init__(self, pgs.driver)
        self.pgs = pgs
        self.ten = ten
        self.chosen = None
        
        rect = pygame.Rect(10,10, 0, 0)
        mainbox = engine.Box(rect)
        
        x = 13
        y = 13
        
        intro = engine.Label(title)
        intro.rect.topleft = (x, y)
        w = intro.rect.w
        y += intro.rect.h + 5
        
        mainbox.add(intro)
        
        self.labels = []
        
        for index in range(10):
            try:
                skill = self.ten[index]
            except IndexError:
                skill = None
            texindex =  ((index + 1) % 10)
            text = "%d: " % texindex
            if skill:
                text += str(skill)
            label = engine.Button(text, self.assign, str(texindex))
            label.rect.topleft = [ x, y ]
            if label.rect.w > w: w = label.rect.w
            y += label.rect.h + 3
            mainbox.add(label)
            self.labels.append(label)
            
        y += 5
        cancelButton = engine.Button("(c)ancel", self.cancel, 'c')
        cancelButton.rect.topleft = [x,y]
        mainbox.add(cancelButton)
        y += cancelButton.rect.h + 3
            
        mainbox.rect.w = w + 6
        mainbox.rect.h = y - rect.y 
                    
        self.add(mainbox)
        
    def assign(self, button):
        self.chosen = self.labels.index(button)
        self.cancel(button)
    
    def cancel(self, button):
        self.driver.pop()
        
    def paint(self, screen):
        self.pgs.paint(screen)
        engine.State.paint(self,screen)
  
class InventoryState(engine.State):
    def __init__(self, pgs, page=0):
        engine.State.__init__(self, pgs.driver)
        self._active = None
        self.aboutToAssign = None
        
        self.pgs = pgs
        self.page = page
        w,h = pgs.driver.screen.getSize()
        
        rect = pygame.Rect( 20, 20, w-40,h-40 )
        mainbox = engine.Box(rect)
        
        y = rect.y + 3
        self.reportLabel = engine.Label(" ")
        self.reportLabel.color = (255,255,255)
        self.reportLabel.rect.topleft = [rect.x + 3, y]
        mainbox.add(self.reportLabel)
        y = self.reportLabel.rect.bottom + 3
        
        equipLabel = engine.Label("Equipped")
        equipLabel.rect.topleft = [ rect.x + 3, y]
        mainbox.add(equipLabel)
        
        inventoryLabel = engine.Label("Inventory")
        inventoryLabel.rect.topleft = [ w/2 + 3, y]
        mainbox.add(inventoryLabel)
        
        # Eqipment
        y = equipLabel.rect.bottom + 10
        # For now, this is hard-coded.  May want to change later
        self.wpnLabel = engine.Label("Weapon: %s" % pgs.pc.weapon)
        self.wpnLabel.rect.topleft = [ rect.x + 3, y ]
        y = self.wpnLabel.rect.bottom + 3
        mainbox.add(self.wpnLabel)
        
        armorLabel = engine.Label("Armor:")
        armorLabel.rect.topleft = [ rect.x + 3, y ]
        y = armorLabel.rect.bottom + 3
        mainbox.add(armorLabel)
        
        self.armorLabels = {}
        for slot in pgs.pc.armor.keys():
            armor = pgs.pc.armor[slot]
            label = engine.Label("  %s : %s" % (slot, armor))
            label.rect.topleft = [ rect.x + 3, y ]
            y = label.rect.bottom + 3
            mainbox.add(label)
            self.armorLabels[slot] = label
            
        if not pgs.pc.armor:
            label = engine.Label("  None")
            label.rect.topleft = [ rect.x + 3, y]
            mainbox.add(label)
            self.armorLabels['None'] = label
            y = label.rect.bottom + 3
        
        # Display inventory
        y = equipLabel.rect.bottom + 10
        self.inventoryLabels = []
        for index, x in enumerate(data.hotkeys):
            name = ""
            index += page * 10
            if index < len(pgs.pc.inventory):
                name = pgs.pc.inventory[index]
            label = engine.Button("%d: %s" % (x, name),self.inventory,str(x))
            label.rect.topleft = [ w/2 + 3, y]
            self.inventoryLabels.append(label)
            mainbox.add(label)
            y = label.rect.bottom + 3
            
        prev = engine.Button("(p)rev", self.prev,'p')
        prev.rect.topleft = [ w/2 + 3, y]
        mainbox.add(prev)
        
        next = engine.Button("(n)ext", self.next, 'n')
        next.rect.topleft = [prev.rect.right + 3, y]
        mainbox.add(next)
        y += next.rect.height + 3
        
        # Options
        y += 10
        exitButton = engine.Button("ex(i)t", self.exit,'i')
        exitButton.rect.topleft = [ rect.x + 3, y ]
        mainbox.add(exitButton)
        
        x = exitButton.rect.right + 3
        equipButton = engine.Button("(e)quip", self.equip,'e')
        equipButton.rect.topleft = [ x, y ]
        mainbox.add(equipButton)
        self.equipButton = equipButton
        
        x = equipButton.rect.right + 3
        assignButton = engine.Button("(a)ssign to hotkey", self.assign,'a')
        assignButton.rect.topleft = [x, y]
        mainbox.add(assignButton)
        self.assignButton = assignButton
        
        x = assignButton.rect.right + 3
        dropButton = engine.Button("(d)rop", self.drop, 'd')
        dropButton.rect.topleft = [x,y]
        mainbox.add(dropButton)
        self.dropButton = dropButton
        
        y = exitButton.rect.bottom + 3
        
        mainbox.rect.height = y - mainbox.rect.y
        
        self.add(mainbox)
 
    def activate(self):
        if self.aboutToAssign != None:
            assigned = self.which.chosen
            if assigned != None:
                item = self.pgs.pc.inventory[self.aboutToAssign]
                skill = items.UseItemSkill("Use %s" % item.name,
                                          self.pgs.pc,
                                          item)
                self.pgs.hotkeys[assigned] = skill
                data = (skill, (assigned + 1) % 10 )
                self.reportLabel.text = "Assigned %s to hotkey %d" % data
                
        self.active = None
        self.aboutToAssign = None
 
    def assign(self, button):
        active = self.active
        if active and active in self.inventoryLabels:
            index = self.getPickedInventory(active)
            if index != None:
                item = self.pgs.pc.inventory[index]
                if item.useable:
                    self.aboutToAssign = index
                    self.which = ChooseFromTenState(self.pgs, self.pgs.hotkeys,
                        "Choose which hotkey to assign")
                    self.driver.push(self.which)
            else:
                self.active = None
        else:
            self.active = button
 
    def drop(self, button):
        active = self.active
        if active and active in self.inventoryLabels:
            index = self.getPickedInventory(active)
            if index != None:
                item = self.pgs.pc.inventory[index]
                tilemap = self.pgs.tiles
                loot = rpg.Loot(tilemap, item)
                loot.world.center = self.pgs.pc.world.center
                tilemap.addLoot(loot)
                self.pgs.pc.deequip(item)
                self.pgs.pc.inventory.remove(item)
                self.refresh()
        else:
            self.active = button
 
    def equip(self, button):
        active = self.active
        if active and active in self.inventoryLabels:
            index = self.getPickedInventory(active)
            if index != None:
                self.pgs.pc.equip(self.pgs.pc.inventory[index])
                self.refresh()
            else:
                self.active = None
        else:
            self.active = button
        
    def exit(self, button):
        self.driver.pop()
 
    def getActive(self):
        return self._active
    
    def getPickedInventory(self, active):
        index = int(active.text[0])
        if index == 0: index = 10
        index -= 1
        index += self.page * 10
        if index < len(self.pgs.pc.inventory):
            return index
        else:
            return None
        
    def inventory(self, button):
        oldActive = self._active
        self.active = button
        if oldActive == self.equipButton:
            self.equip(button)
        elif oldActive == self.assignButton:
            self.assign(button)
        elif oldActive == self.dropButton:
            self.drop(button)
        
    def next(self, button):
        self.page += 1
        self.refresh()
        
    def paint(self, screen):
        self.pgs.paint(screen)
        engine.State.paint(self,screen)
        
    def prev(self, button):
        self.page = max(0, self.page-1)
        self.refresh()
        
    def refresh(self):
        self.driver.pop()
        self.driver.push(InventoryState(self.pgs,self.page))
        
    def setActive(self, button):
        if self._active:
            self._active.background = None
        self._active = button
        if button: 
            button.background = ( 64, 64, 64 )
        
    active = property(getActive, setActive)
    
class GameOverState(engine.State):
    def __init__(self,pgs):
        engine.State.__init__(self, pgs.driver)
        
        rip = engine.Label("R.I.P. %s" % pgs.pc.name, (255, 64, 64), 32)
        rip.rect.topleft = (10,10)
        self.add(rip)
        y = rip.rect.bottom + 10
        x = 10
        
        desc = "A level %d %s, mercilessly slain %d stories above the earth"
        data = (pgs.pc.level, pgs.pc.progression.name, pgs.level)
        
        descLabel = engine.Label(desc % data)
        descLabel.rect.topleft = [x,y]
        self.add(descLabel)
        
        gameover = engine.Label("Game Over", (255, 255, 255), 64)
        gameover.rect.center = pgs.driver.screen.center
        self.add(gameover)
        
        quit = engine.Button("(q)uit", self.quit, 'q')
        quit.rect.bottomleft = [ 3, pgs.driver.screen.h - 3]
        self.add(quit)
        
    def quit(self, button):
        raise SystemExit
    
class ConfirmDialog(engine.State):
    def __init__(self, driver, message):
        engine.State.__init__(self, driver)
        
        box = engine.Box(pygame.Rect(0, 0, 0, 0))
        self.add(box)
        msg = engine.Label(message)
        box.add(msg)
        w = msg.rect.w
        
        yes = engine.Button("(y)es", self.yes,'y')
        if yes.rect.w > w: w = yes.rect.w
        box.add(yes)
        no = engine.Button("(n)o", self.no,'n')
        box.add(no)
        
        box.rect.w = w + 6
        box.rect.h = no.rect.h + msg.rect.h + 9
        box.rect.center = driver.screen.center
        
        msg.rect.topleft = ( box.rect.x + 3, box.rect.y + 3 )
        yes.rect.topleft = ( box.rect.x + 3, msg.rect.bottom + 3)
        no.rect.topleft = ( box.rect.x + box.rect.w - no.rect.w - 3,
                            msg.rect.bottom+3)
        
    def yes(self, button):
        self.confirmed = True
        self.driver.pop()
        
    def no(self, button):
        self.confirmed = False
        self.driver.pop()  